//
//  LiftSectionTableViewController.swift
//  DragAndDrop
//
//  Created by Toshihiro Morimoto on 2017/09/10.
//  Copyright © 2017 Toshihiro Morimoto. All rights reserved.
//

import UIKit

class LiftSectionTableViewController: UITableViewController {
    
    let images = [#imageLiteral(resourceName: "scene_1"), #imageLiteral(resourceName: "scene_2"), #imageLiteral(resourceName: "scene_3"), #imageLiteral(resourceName: "scene_4"), #imageLiteral(resourceName: "scene_5")]

    override func viewDidLoad() {
        super.viewDidLoad()

        tableView.dragDelegate = self
        tableView.dragInteractionEnabled = true
        tableView.isSpringLoaded = true
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return images.count
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "LiftSectionTableViewCell", for: indexPath) as! LiftSectionTableViewCell
        cell.iconImageView.image = images[indexPath.row]
        return cell
    }
    
    // MARK: - Table view delegate
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return view.bounds.width
    }
}

extension LiftSectionTableViewController: UITableViewDragDelegate {
    func tableView(_ tableView: UITableView, itemsForBeginning session: UIDragSession, at indexPath: IndexPath) -> [UIDragItem] {
        return makeDragItems(for: indexPath)
    }
    
    func tableView(_ tableView: UITableView, itemsForAddingTo session: UIDragSession, at indexPath: IndexPath, point: CGPoint) -> [UIDragItem] {
        return makeDragItems(for: indexPath)
    }
    
    func tableView(_ tableView: UITableView, dragPreviewParametersForRowAt indexPath: IndexPath) -> UIDragPreviewParameters? {
        // ドラッグ時の画像プレビューを変更できます
        let parameters = UIDragPreviewParameters()
        let width = tableView.bounds.width
        let rect = CGRect(x: 0,y: 0, width: width, height: width)
        parameters.visiblePath = UIBezierPath(roundedRect: rect, cornerRadius: width / 2)
        return parameters
    }
    
    private func makeDragItems(for indexPath: IndexPath) -> [UIDragItem] {
        let image = images[indexPath.row]
        let itemProvider = NSItemProvider(object: image)
        let dragItem = UIDragItem(itemProvider: itemProvider)
        dragItem.localObject = image
        return [dragItem]
    }
}

class LiftSectionTableViewCell: UITableViewCell {
    @IBOutlet weak var iconImageView: UIImageView!
}

